// PIC16F88

#define LED1 PORTB.F2

#define BTN_Up      PORTA.F0
#define BTN_Down    PORTA.F1
#define BTN_ReSend  PORTB.F5

#define Lo(param) ((char *)&param)[0]
#define Hi(param) ((char *)&param)[1]

// Software I2C connections
sbit Soft_I2C_Scl           at RB4_bit;
sbit Soft_I2C_Sda           at RB1_bit;
sbit Soft_I2C_Scl_Direction at TRISB4_bit;
sbit Soft_I2C_Sda_Direction at TRISB1_bit;
// End Software I2C connections

#define ADR 0b00101110
//            76543210

int freq_index = 0;

void Config(){
    OSCCON  = 0b01110010;
    //          76543210

    PORTA  = 0b00000000;
    ANSEL  = 0b00000000;
    TRISA  = 0b00000011;
    //         76543210

    PORTB = 0b00000000;
    TRISB = 0b00100000;
    //        76543210

    OPTION_REG = 0b00000000;
    INTCON     = 0b00000000;   // INT0IE: RB0/INT External Interrupt Enable bit
    //             76543210
}

void LTC6904_SendFrequency(unsigned int freq){
     LED1 = 1;
     
     // Send frequency for LTC6904
     Soft_I2C_Start();
     Soft_I2C_Write(ADR);
     Soft_I2C_Write(Hi(freq));
     Soft_I2C_Write(Lo(freq));
     Soft_I2C_Stop();

     delay_ms(300);
     LED1 = 0;
}

void set_frequency(unsigned int OCT, unsigned long l_freq){
     unsigned int DAC = 0;
     unsigned int freq = 0;
     
     DAC = 2048 - 2078 * pow(2, 10 + OCT) / l_freq;
     DAC = DAC & 0x3FF;

     freq = (OCT << 12) | (DAC << 2);
     
     LTC6904_SendFrequency(freq);
}

void main() {
     unsigned char send_data = 1;
     Config();

     LED1 = 0;

     Soft_I2C_Init();
     Delay_ms(500);
     freq_index = 0;

     do{
        if(BTN_Up == 0) {
            LED1 = 1;
            if(freq_index < 12) freq_index++;
            send_data = 1;
            
            delay_ms(500);
            LED1 = 0;
            
            while(BTN_Up == 0);
        }
        
        if(BTN_Down == 0) {
            LED1 = 1;
            if(freq_index > 0) freq_index--;
            send_data = 1;

            delay_ms(500);
            LED1 = 0;
            
            while(BTN_Down == 0);
        }

        if(BTN_ReSend == 0) {
            LED1 = 1;
            send_data = 1;

            delay_ms(500);
            LED1 = 0;

            while(BTN_ReSend == 0);
        }
        
        if(send_data) {
            switch(freq_index) {
                case 0:  set_frequency(3,  10000UL);         break; // 10 KHz
                case 1:  set_frequency(5,  50000UL);         break; // 50 KHz
                case 2:  set_frequency(6,  100000UL);        break; // 100 KHz
                case 3:  set_frequency(8,  500000UL);        break; // 500 KHz
                case 4:  set_frequency(9,  1000000UL);       break; // 1 MHz
                case 5:  set_frequency(10, 2000000UL);       break; // 2 MHz
                case 6:  set_frequency(12, 5000000UL);       break; // 5 MHz
                case 7:  set_frequency(13, 10000000UL);      break; // 10 MHz
                case 8:  set_frequency(14, 20000000UL);      break; // 20 MHz
                case 9:  set_frequency(14, 30000000UL);      break; // 30 MHz
                case 10: set_frequency(15, 40000000UL);      break; // 40 MHz
                case 11: set_frequency(15, 50000000UL);      break; // 50 MHz
                case 12: set_frequency(15, 60000000UL);      break; // 60 MHz
                default: set_frequency(0,  2000UL);          break; // 2 KHz
            }
        }
        
        send_data = 0;
     } while(1);
}